import es
import vecmath
import gamethread
import playerlib

info = es.AddonInfo()
info.name = 'Class Teleporter'
info.basename = 'class_teleporter'
info.description = 'Allows medics and engineers to teleport to objects needing their attention'
info.tags = 'tf2'

gtdn = 'class_teleport_medic_%d'

need_medic = {2: set(), 3: set()}

CLASS_MEDIC = 5
CLASS_ENGINEER = 91 # change to 9 when implemented

cvar_medic_initial = es.ServerVar('ct_medic_initial', '0', 'Initial teleporting cost')
cvar_medic_increment = es.ServerVar('ct_medic_increment', '0.015', 'Cost increment on distance')
cvar_engineer_initial = es.ServerVar('ct_engineer_initial', '0', 'Initial teleporting cost')
cvar_engineer_increment = es.ServerVar('ct_engineer_increment', '0.030', 'Cost increment on distance')
cvar_medic_shouttime = es.ServerVar('ct_medic_shouttime', '5', 'Time in seconds the MEDIC shout is in effect')

def load():
    es.addons.registerClientCommandFilter(cff)

def unload():
    es.addons.unregisterClientCommandFilter(cff)

def player_death(event_var):
    userid = int(event_var['userid'])
    teamid = int(event_var['es_userteam'])
    delMedicList(need_medic[teamid], userid)

def player_spawn(event_var):
    userid = int(event_var['userid'])
    player = playerlib.getPlayer(userid)
    classid = getClassId(player)
    if classid == CLASS_MEDIC:
        medicIntro(userid)
    elif classid == CLASS_ENGINEER:
        engineerIntro(userid)
    

def cff(userid, args):
    if args == ['voicemenu', '0', '0']:
        # Someone shouted MEDIC!
        player = playerlib.getPlayer(userid)
        teamid = int(player.get('teamid'))
        if teamid in need_medic:
            addMedicList(need_medic[teamid], userid)
    elif args[0] == 'lastdisguise':
        # Someone pressed B, check the player class
        player = playerlib.getPlayer(userid)
        teamid = int(player.get('teamid'))
        if teamid in need_medic:
            #es.msg('getclassid')
            classid = getClassId(player)
            #es.msg('done')
            if classid == CLASS_MEDIC:
                #es.msg('medicusecommand')
                medicUseCommand(player)
            elif classid == CLASS_ENGINEER:
                engineerUseCommand(player)
    return True

def addMedicList(l, userid):
    if userid not in l:
        l.add(userid)
    else:
        gamethread.cancelDelayed(gtdn%userid)
    gamethread.delayedname(int(cvar_medic_shouttime), gtdn%userid, delMedicList, (l, userid))

def delMedicList(l, userid):
    if userid in l:
        l.remove(userid)
        gamethread.cancelDelayed(gtdn%userid)

def getClassId(player):
    manager = es.getentityindex('tf_player_manager')
    index = int(player.get('index'))
    return int(
        es.getindexprop(manager, "CTFPlayerResource.m_iPlayerClass.%03d"%index)
        )

def medicIntro(userid):
    es.tell(userid, '[MEDIC] Press "b" to \xc3\x9cberTeleport')

def engineerIntro(userid):
    es.tell(userid, '[ENGINEER] Press "b" to MetalTeleport')

def medicUseCommand(player):
    teamid = int(player.get('teamid'))
    l = need_medic[teamid]
    if len(l):
        mylocation = vecmath.vector(player.get('location'))
        maxdistance = 16384
        minplayer = 0
        #es.msg('for uid in l')
        for uid in l:
            if uid == int(player):
                # can't teleport to self
                continue
            targetlocation = vecmath.vector(es.getplayerlocation(uid))
            distance = vecmath.distance(mylocation, targetlocation)
            if distance < maxdistance:
                maxdistance = distance
                minplayer = uid
                minlocation = targetlocation
        #es.msg('if minplayer')
        if minplayer:
            #es.msg('cost')
            cost = float(cvar_medic_initial) + distance*float(cvar_medic_increment)
            #es.msg('medigun = find...')
            medigun = findWeapon('tf_weapon_medigun', player)
            #es.msg('uberlevle = getUber...')
            uberlevel = getUberCharge(medigun)
            #es.msg('cost = %f, uberlevel = %f'%(cost,uberlevel))
            if cost > uberlevel:
                es.tell(int(player), 'You don\'t have enough \xc3\x9cberCharge to teleport')
                return False
            else:
                #es.msg('setubercharge')
                setUberCharge(medigun, uberlevel - cost)
                #es.msg('eyes')
                es.setplayerprop(int(player), 'CBaseEntity.m_vecOrigin', str(minlocation))
                eyes_x = es.getplayerprop(int(player), 'CTFPlayer.tfnonlocaldata.m_angEyeAngles[1]')
                eyes_y = es.getplayerprop(minplayer, 'CTFPlayer.tfnonlocaldata.m_angEyeAngles[1]')
                es.msg('value = '+str(eyes_x)+","+str(eyes_y))
                es.setang(int(player), eyes_x, eyes_y)
                return True
    es.tell(int(player), 'No users requesting medical attention.')
            

def findWeapon(classname, player):
    ents = es.createentitylist(classname)
    plhandle = int(player.get('handle'))
    for index in ents:
        handle = int(es.getindexprop(index, 'CBaseCombatWeapon.m_hOwner'))
        if handle == plhandle:
            return index
    return 0

def getUberCharge(index):
    basecharge = float(es.getindexprop(index, 'CWeaponMedigun.m_flChargeLevel'))
    return basecharge*100.0

def setUberCharge(index, value):
    es.setindexprop(index, 'CWeaponMedigun.m_flChargeLevel', value/100.0)
    return value/100.0

